@charset "UTF-8";
/* ado.css */
@font-face
{
  font-family: FreeSerif;
  src: url(../fonts/FreeSerif-Glagolitic-UC.woff);
}
body{
  font-family: 'Ubuntu', sans-serif;
  margin: 0;
  color: #46400D;
  background-color: #fff;
  background-image: url(ui-bg_diagonal-maze_light_10x10.png);
}
h1,h2,h3,h4 {
  margin-bottom: 0.5em;
  margin-top: 1em;
}
h1 {
  margin-left: 1em;
}
h2 {
  margin-left: 2em;
}
h3 {
  margin-left: 3em;
}

h4 {
  margin-left: 4em;
}

h1,h2,h3,h4, ui a {
  text-shadow: 2px 2px 2px #aaa;  
}

#adobar {
  /* box-shadow: none|h-shadow v-shadow blur spread color |inset|initial|inherit; */
    box-shadow: 0px 1px 5px #222;
}
.footer,
.main.container {
  position: relative;
  margin: auto;
  min-width: 460px;
  width: 98%;
  max-width: 1024px;
  box-shadow: 0px 0px 5px #aaa;
  background: #FAFAFA;
  border-radius: 0.25em;
  /* top right bottom left */
  padding: 2em 1em 1em 1em;
  font-size: 1.3em;
}

aside#toc ul {
  padding: 0;
  margin: 0;
}

aside#toc li {
  list-style: none inside none;
  padding: 0 0 0 1.2ex;
}
article.main.container li {
  margin-bottom:0.5em ;
}
/*
aside#toc li a {
  font-size: 1.1em;
}
*/
a {
  color: rgb(0, 159, 218);
  text-decoration: none;
}

a.footnote {
  vertical-align: super;
  font-size: 0.8em;
  font-weight: bolder;
}

a.icon.ado.popup.visible {
  font-family: FreeSerif;  
}
i.icon.ado{
  font-family: FreeSerif;
  text-shadow:2px 2px 2px #aaa;
  font-weight: bolder;
}
i.icon.ado:before {
  content: "Ⰷ";
}

.тц,.ne,.ъ,.ь {display: none; visibility: hidden;}

.ui.hidden {
  display: none;
}
/* box-shadow: none|h-shadow v-shadow blur spread color |inset|initial|inherit; */
#ado-img {
  box-shadow: 1px 1px 0.1em 0.1em #aaa;
  border: 1px #aaa outset;
}
.small {font-size: small}
code {
  color: #993;
  white-space: nowrap;
}
section.login_form {
  padding-top: 2em;
  margin: auto;
  min-width: 300px;
  width: 80%;
  max-width: 600px;
}