package Ado::Control::Ado;
use Mojo::Base 'Ado::Control';


1;

=pod

=encoding utf8

=head1 NAME

Ado::Control::Ado - Base controller class for L<Ado> back-office applications. 

=head1 SYNOPSIS

package Ado::Control::Ado::MyAdoApp;
use Mojo::Base 'Ado::Control::Ado';
#your code here
1;

=head1 DESCRIPTION

Ado::Control::Ado is the base controller class for all back-office controllers.

=head1 ATTRIBUTES

Ado::Control::Ado inherits all the attributes from 
<Ado::Control> and defines attributes shared among all back-office controllers.




=head1 METHODS

Ado::Control::Ado inherits all methods from L<Ado::Control::Ado> and 
implements the following new ones.


=head1 SPONSORS

The original author

=head1 SEE ALSO

L<Ado::Control>, L<Mojolicious::Controller>, L<Mojolicious::Guides::Growing/Model_View_Controller>,
L<Mojolicious::Guides::Growing/Controller_class>


=head1 AUTHOR

Красимир Беров (Krasimir Berov)

=head1 COPYRIGHT AND LICENSE

Copyright 2013-2014 Красимир Беров (Krasimir Berov).

This program is free software, you can redistribute it and/or
modify it under the terms of the 
GNU Lesser General Public License v3 (LGPL-3.0).
You may copy, distribute and modify the software provided that 
modifications are open source. However, software that includes 
the license may release under a different license.

See http://opensource.org/licenses/lgpl-3.0.html for more information.

=cut

